/*
  Uno sketch to control an STI6606z driver chip (AX1201728 also) for VID2905 or X27168 instrument
  stepper motor. In setup(), the motor moves to the negative limit, sets zero
  there, then goes nearly full scale and back to almost zero. Then a pot is used
  to move the motor for testing and demonstration.
  The AccelStepper library is used. The STI6606z produces 1/12 step microsteps.
  10/7/22              --jkl
*/
// Include the AccelStepper Library
#include <AccelStepper.h>
#include <elapsedMillis.h>


// Motor Connections (constant current, step/direction bipolar motor driver)
const int enabPin = 4;
const int dirPin = 6;
const int stepPin = 5;
// This defines the analog input pin for reading the control voltage
// Tested with a 10k linear pot between 5v and GND
const int potValue = A0;

// modified for my stepper and controller
AccelStepper myStepper(AccelStepper::DRIVER, stepPin, dirPin);           // works for STI6606z (Bipolar, constant current, step/direction driver)

elapsedMillis printTime;
const int updateTime = 100;   // update time in milliseconds. Tune as desired. 100 = every tenth second.

const long STEPS_PER_REVOLUTION = 315 * 12;   // for 315 degrees with twelve step microstepping
//const float STEPS_PER_DEGREE =12.0;    // for 1/12 degree per step

// pot reading must move out of this band to cause motion - avoid judder
#define DEADBAND 5

void setup() {
  //Serial.begin(115200);
  pinMode(enabPin,OUTPUT);
  digitalWrite(enabPin,LOW); 
  delay(100);
  digitalWrite(enabPin,HIGH);    // this enables the controller
  myStepper.setMaxSpeed(2000);
  myStepper.setAcceleration(2400);
  myStepper.runToNewPosition(-4000);   // runs the stepper backwards to 'zero'
  myStepper.setCurrentPosition(0);
  myStepper.runToNewPosition(3000);
  myStepper.runToNewPosition(100);
  delay(1000);

  printTime = 0;
  /* Uncomment to print messages
  Serial.print(printTime);
  Serial.println("  Starting");
  */
}

void loop() {
  static int oldReading = 0;  // old ADC reading
  //static bool printOnce = false;   // print flag
  if (printTime >= updateTime) {
    printTime = 0;
    // Read new position  - four averages to cut noise.
    int analog_in = (analogRead(potValue) + analogRead(potValue) + analogRead(potValue) + analogRead(potValue)) / 4;
    if ((analog_in > (oldReading + DEADBAND)) || (analog_in < (oldReading - DEADBAND))) {
      // map it to a the maximum speed range
      myStepper.moveTo((long)(map(analog_in, 0, 1023, 1, STEPS_PER_REVOLUTION)));
      oldReading = analog_in;
      //printOnce = true;
    }
  }
  //if (myStepper.distanceToGo() != 0) {
    myStepper.run();
  //} 
  /* Uncomment toprint info
  else if (printOnce == true) {
    Serial.print(oldReading);
    Serial.print ("   ");
    Serial.print(myStepper.currentPosition());
    Serial.print ("   ");
    Serial.println(myStepper.targetPosition());
    printOnce = false;
  }*/
}
